<?php
/**
 * @package      cb_myprojects
 *
 * @author       Tobias Kuhn (eaxs)
 * @copyright    Copyright (C) 2013 Tobias Kuhn. All rights reserved.
 * @license      http://www.gnu.org/licenses/gpl.html GNU/GPL, see LICENSE.txt
 */


if (! (defined( '_VALID_CB' ) || defined( '_JEXEC' ) || defined( '_VALID_MOS' )) ) {
    die();
}


class PFprojectsTab extends cbTabHandler
{
    /**
     * Constructor
     *
     */
    public function PFprojectsTab()
    {
        $this->cbTabHandler();
    }


    /**
     * Renders a tab
     *
     * @param     ?         $tab     ?
     * @param     object    $user    The user we are currently looking at
     * @param     ?         $ui      ?
     *
     * @return    string    $html    The generated html
     */
    public function getDisplayTab($tab, $user, $ui)
    {
        $html = "";

        // Make sure the component exists
        if (!PFprojectsTab::componentExists()) {
            return $html;
        }

        // Include the projects router
        jimport('projectfork.library');
        require_once JPATH_SITE . '/components/com_pfprojects/helpers/route.php';

        // Count projects
        $count = PFprojectsTab::getCount($user->id, $this->params);

        if ($count == 0) {
            return $html;
        }

        $html = PFprojectsTab::getProjectsHTML($user->id, $this->params);

        return $html;
    }


    /**
     * Renders a list of Projectfork projects
     *
     * @param     integer    $userid    The user id of the current profile
     * @param     object     $params    The plugin params
     *
     * @return    string     $html      The project list HTML
     */
    public static function getProjectsHTML($userid, $params)
    {
        // Get the projects
        $items = PFprojectsTab::getItems($userid, $params);

        // Get authorised view levels
        $levels = JFactory::getUser()->getAuthorisedViewLevels();

        // Get plugin parameters
        $filter_shownoauth = (int) $params->get('show_unauthorised', 0);
        $show_deadline     = (int) $params->get('show_deadline', 1);
        $show_author       = (int) $params->get('show_author', 1);
        $text_limit        = (int) $params->get('limit', 50);
        $date_format       = $params->get('date_format', 'd-m-Y');
        $null_date         = JFactory::getDbo()->getNullDate();
        $itemid            = JRequest::getUInt('Itemid');

        $html  = '<table id="pfprojectstab" class="category table table-striped table-bordered table-hover">';
        $html  .= '<thead>';
        $html  .= '<tr>';
        $html  .= '<th>Project</th>';
        $html  .= '<th>Progress</th>';
        if ($show_author)   $html  .= '<th>Created By</th>';
        if ($show_deadline) $html  .= '<th>Deadline</th>';
        $html  .= '</tr>';
        $html  .= '</thead>';
        $html  .= '<tbody>';

        $k = 0;

        foreach($items AS $i => $item)
        {
            // Determine whether the user can access the current project or not
            $can_view = (($filter_shownoauth && in_array($item->access, $levels)) || !$filter_shownoauth);

            $html .= '<tr class="cat-list-row' . $k . '">';

            $output = $item->title;

            if ($can_view) {
                $link = PFprojectsHelperRoute::getDashboardRoute($item->id . ':' . $item->alias);
                $output = '<a href="' . $link . '">' . $item->title . '</a>';
            }

            $html .= '<td>' . $output . '</td>';
            $html .= '<td>' . $item->progress . '%</td>';

            if ($show_author) {
                $output = '';

                if ($item->name != '') {
                    $profile_link = JRoute::_('index.php?option=com_comprofiler&task=userProfile&user=' . $item->created_by . '&Itemid='. $itemid);
                    $output = '<a href="' . $profile_link . '">' . $item->username . '</a>';
                }

                $html .= '<td>' . $output . '</td>';
            }

            if ($show_deadline) {
                $output = '';

                if ($item->end_date != $null_date) {
                    $date   = new JDate($item->end_date);
                    $output = $date->format($date_format);
                }

                $html .= '<td>' . $output . '</td>';
            }

            $html .= '</tr>';

            $k = 1 - $k;
        }


        $html  .= '</tbody>';
        $html  .= '</table>';



        return $html;
    }


    /**
     * Checks if the Projectfork Projects component is installed
     *
     * @return    boolean    True if exists, False if not
     */
    public static function componentExists()
    {
        $db    = JFactory::getDbo();
        $query = $db->getQuery(true);

        // Check if Projects component exists
        $query->select('extension_id')
              ->from('#__extensions')
              ->where('name = ' . $db->quote('com_pfprojects'));

        $db->setQuery($query);

        return ($db->loadResult() ? true : false);
    }


    /**
     * Fetches a list of project records from the database
     *
     * @param     integer    $userid    The user id of the current profile
     * @param     object     $params    The plugin params
     *
     * @return    array      $items     The project list
     */
    public static function getItems($userid, $params)
    {
        $db    = JFactory::getDbo();
        $query = $db->getQuery(true);

        // Get the profile user
        $profile_user   = JFactory::getUser($userid);
        $profile_levels = $profile_user->getAuthorisedViewLevels();

        // Get the viewing user
        $user   = JFactory::getUser();
        $levels = $user->getAuthorisedViewLevels();

        // Get the common viewing levels
        $common_levels = array_intersect($profile_levels, $levels);

        // Get possible filters
        $filter_created    = (int) $params->get('show_created_only', 0);
        $filter_shownoauth = (int) $params->get('show_unauthorised', 0);
        $limit             = (int) $params->get('count', 10);

        // Get a list of projects
        $query->select('a.id, a.title, a.alias, a.description, a.created_by, a.access, a.end_date')
              ->from('#__pf_projects AS a');

        $query->select('u.name, u.username')
              ->join('left', '#__users AS u ON u.id = a.created_by');

        // Filter by pub state
        $query->where('a.state = 1');

        // Filter by author?
        if ($filter_created) {
            $query->where('a.created_by = ' . (int) $userid);

            // Filter by viewing access
            if (!$filter_shownoauth) {
                $query->where('a.access IN(' . implode(', ', $levels) . ')');
            }
        }
        else {
            if (!$filter_shownoauth) {
                $query->where('a.access IN(' . implode(', ', $common_levels) . ')');
            }
            else {
                $query->where('a.access IN(' . implode(', ', $profile_levels) . ')');
            }
        }

        $sort  = $params->get('sort', 'a.title');
        $order = $params->get('order', 'ASC');

        $query->order($db->escape($sort . ' ' . $order));

        $db->setQuery($query, 0, $limit);
        $items = $db->loadObjectList();

        if (empty($items) || !is_array($items)) {
            $items = array();

            return $items;
        }

        $pks      = JArrayHelper::getColumn($items, 'id');
        $progress = PFprojectsTab::getProgress($pks);

        foreach ($items as $i => $item)
        {
            $items[$i]->progress = (isset($progress[$item->id]) ? $progress[$item->id] : 0);
        }

        return $items;
    }


    /**
     * Returns the amount of projects
     *
     * @param     integer    $userid    The user id of the current profile
     * @param     object     $params    The plugin params
     *
     * @return    integer    $count     The project count
     */
    public static function getCount($userid, $params)
    {
        $db    = JFactory::getDbo();
        $query = $db->getQuery(true);

        // Get the profile user
        $profile_user   = JFactory::getUser($userid);
        $profile_levels = $profile_user->getAuthorisedViewLevels();

        // Get the viewing user
        $user   = JFactory::getUser();
        $levels = $user->getAuthorisedViewLevels();

        // Get the common viewing levels
        $common_levels = array_intersect($profile_levels, $levels);

        // Get possible filters
        $filter_created    = (int) $params->get('show_created_only', 0);
        $filter_shownoauth = (int) $params->get('show_unauthorised', 0);

        // Get a list of projects
        $query->select('COUNT(a.id)')
              ->from('#__pf_projects AS a');

        // Filter by pub state
        $query->where('a.state = 1');

        // Filter by author?
        if ($filter_created) {
            $query->where('a.created_by = ' . (int) $userid);

            // Filter by viewing access
            if (!$filter_shownoauth) {
                $query->where('a.access IN(' . implode(', ', $levels) . ')');
            }
        }
        else {
            if (!$filter_shownoauth) {
                $query->where('a.access IN(' . implode(', ', $common_levels) . ')');
            }
            else {
                $query->where('a.access IN(' . implode(', ', $profile_levels) . ')');
            }
        }

        $db->setQuery($query);
        $count = (int) $db->loadResult();

        return $count;
    }


    /**
     * Returns the total progress for a set of projects
     *
     * @param     array    $pks     The projects to get the progress for
     *
     * @return    array    $prog    The progress for the projects
     */
    public static function getProgress($pks)
    {
        $db    = JFactory::getDbo();
        $query = $db->getQuery(true);
        $prog  = array();

        // Get total task count
        $query->select('project_id, COUNT(id) AS total')
              ->from('#__pf_tasks')
              ->where('project_id IN(' . implode(', ', $pks) . ')')
              ->where('state != -2')
              ->group('project_id')
              ->order('id ASC');

        $db->setQuery($query);
        $total = $db->loadAssocList('project_id', 'total');

        // Get completed tasks count
        $query->clear();
        $query->select('project_id, COUNT(id) AS total')
              ->from('#__pf_tasks')
              ->where('project_id IN(' . implode(', ', $pks) . ')')
              ->where('state != -2')
              ->where('complete = 1')
              ->group('project_id')
              ->order('id ASC');

        $db->setQuery($query);
        $completed = $db->loadAssocList('project_id', 'total');

        foreach ($pks AS $pk)
        {
            $count_complete = (int) (isset($completed[$pk]) ? $completed[$pk] : 0);
            $count_total    = (int) (isset($total[$pk])     ? $total[$pk]     : 0);

            if (!$count_total || !$count_complete) {
                $progress = 0;
            }
            elseif ($count_complete == $count_total) {
                $progress = 100;
            }
            else {
                $progress = round($count_complete * (100 / $count_total));
            }

            $prog[$pk] = $progress;
        }

        return $prog;
    }
}
?>
